@@ -15,17 +15,14 @@
 # limitations under the License.
 
 def parse(xml_or_file, parser='lxml'):
-    """ Parse xml (string) or file object. This is just an wrapper for GPXParser.parse() function """
+    """
+    Parse xml (string) or file object. This is just an wrapper for 
+    GPXParser.parse() function.
+    """
 
     from . import gpx as mod_gpx
     from . import parser as mod_parser
 
     parser = mod_parser.GPXParser(xml_or_file, parser=parser)
 
-    gpx = parser.parse()
-
-    if not parser.is_valid():
-        raise mod_gpx.GPXException('Error parsing {0}: {1}'.format(xml_or_file[0 : 100], parser.get_error()))
-
-    return gpx
-
+    return parser.parse()
