@@ -36,9 +36,6 @@ class XMLParser:
     Used when lxml is not available. Uses standard minidom.
     """
 
-    dom = None
-    xml = None
-
     def __init__(self, xml):
         self.xml = xml
         self.dom = mod_minidom.parseString(xml)
@@ -92,10 +89,6 @@ class LXMLParser:
     Used when lxml is available. 
     """
 
-    xml = None
-    dom = None
-    ns = None
-
     def __init__(self, xml):
         if mod_utils.PYTHON_VERSION[0] == '3':
             # In python 3 all strings are unicode and for some reason lxml 
@@ -164,13 +157,6 @@ def parse_time(string):
 
 class GPXParser:
 
-    gpx = None
-    xml = None
-    valid = None
-    error = None
-    xml_parser_type = None
-    xml_parser = None
-
     def __init__(self, xml_or_file=None, parser=None):
         """
         Parser may be lxml of minidom. If you set to None then lxml will be used if installed
@@ -179,26 +165,24 @@ class GPXParser:
         self.init(xml_or_file)
         self.gpx = mod_gpx.GPX()
         self.xml_parser_type = parser
+        self.xml_parser = None
 
     def init(self, xml_or_file):
         text = xml_or_file.read() if hasattr(xml_or_file, 'read') else xml_or_file
         self.xml = mod_utils.make_str(text)
-
-        self.valid = False
-        self.error = None
-
         self.gpx = mod_gpx.GPX()
 
-    def is_valid(self):
-        return self.valid
-
-    def get_error(self):
-        return self.error
-
     def get_gpx(self):
         return self.gpx
 
     def parse(self):
+        """
+        Parses the XML file and returns a GPX object.
+
+        It will throw GPXXMLSyntaxException if the XML file is invalid or 
+        GPXException if the XML file is valid but something is wrong with the 
+        GPX data.
+        """
         try:
             if self.xml_parser_type is None:
                 if mod_etree:
@@ -216,11 +200,18 @@ class GPXParser:
 
             return self.gpx
         except Exception as e:
+            # The exception here can be a lxml or minidom exception.
             mod_logging.debug('Error in:\n%s\n-----------\n' % self.xml)
             mod_logging.exception(e)
-            self.error = str(e)
 
-            return None
+            # The library should work in the same way regardless of the 
+            # underlying XML parser that's why the exception thrown 
+            # here is GPXXMLSyntaxException (instead of simply throwing the 
+            # original minidom or lxml exception e). 
+            #
+            # But, if the user need the original exception (lxml or minidom) 
+            # it is available with GPXXMLSyntaxException.original_exception:
+            raise mod_gpx.GPXXMLSyntaxException('Error parsing XML: %s' % str(e), e)
 
     def __parse_dom(self):
 
@@ -446,6 +437,9 @@ class GPXParser:
         comment_node = self.xml_parser.get_first_child(node, 'cmt')
         comment = self.xml_parser.get_node_data(comment_node)
 
+        name_node = self.xml_parser.get_first_child(node, 'name')
+        name = self.xml_parser.get_node_data(name_node)
+
         hdop_node = self.xml_parser.get_first_child(node, 'hdop')
         hdop = mod_utils.to_number(self.xml_parser.get_node_data(hdop_node))
 
@@ -460,7 +454,7 @@ class GPXParser:
 
         return mod_gpx.GPXTrackPoint(latitude=latitude, longitude=longitude, elevation=elevation, time=time,
             symbol=symbol, comment=comment, horizontal_dilution=hdop, vertical_dilution=vdop,
-            position_dilution=pdop, speed=speed)
+            position_dilution=pdop, speed=speed, name=name)
 
 
 
@@ -478,16 +472,13 @@ if __name__ == '__main__':
 
     print(gpx.to_xml())
 
-    if parser.is_valid():
-        print('TRACKS:')
-        for track in gpx.tracks:
-            print('name%s, 2d:%s, 3d:%s' % (track.name, track.length_2d(), track.length_3d()))
-            print('\tTRACK SEGMENTS:')
-            for track_segment in track.segments:
-                print('\t2d:%s, 3d:%s' % (track_segment.length_2d(), track_segment.length_3d()))
+    print('TRACKS:')
+    for track in gpx.tracks:
+        print('name%s, 2d:%s, 3d:%s' % (track.name, track.length_2d(), track.length_3d()))
+        print('\tTRACK SEGMENTS:')
+        for track_segment in track.segments:
+            print('\t2d:%s, 3d:%s' % (track_segment.length_2d(), track_segment.length_3d()))
 
-        print('ROUTES:')
-        for route in gpx.routes:
-            print(route.name)
-    else:
-        print('error: %s' % parser.get_error())
+    print('ROUTES:')
+    for route in gpx.routes:
+        print(route.name)
